package com.hungraim.ltc.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.hungraim.ltc.pojo.entity.assessment.ChRefuseReason;
import com.hungraim.ltc.pojo.entity.disable.ChCfgSrvDisableLevel;
import com.hungraim.ltc.pojo.vo.assessment.*;
import com.hungraim.ltc.service.TaskService;
import com.hungraim.ltc.util.Result;
import com.hungraim.ltc.util.ResultCode;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * 失能评定Controller
 *
 * @author czz
 */
@RestController
@RequestMapping("/api.assessment/task")
@Slf4j
public class TaskController {

    @Autowired
    private TaskService taskService;

    /**
     * 评定任务分配、查询列表
     *
     * @param organId       适用机构代码
     * @param provincial    所在省
     * @param city          所在市
     * @param area          所在区域
     * @param street        所在街道
     * @param realName      失能人员姓名
     * @param certiCode     失能人员身份证号码
     * @param taskName      评定人员姓名
     * @param taskTel       评定人员手机号码
     * @param daysRemaining 评定剩余天数
     * @param status        任务状态
     * @param isReturn      是否退回
     * @param pageNum       当前页
     * @param pageSize      每页几条数据
     */
    @RequestMapping("/taskDistributionList")
    public Result<List<TaskDistributionVO>> taskDistributionList(String organId, String provincial, String city,
                                                                 String street, String realName, String certiCode, String taskName, String taskTel,
                                                                 String daysRemaining, String status, String isReturn, String area,
                                                                 Integer pageNum, Integer pageSize) {
        if (pageNum == null) {
            pageNum = 1;
        }
        if (pageSize == null) {
            pageSize = 10;
        }
        IPage<TaskDistributionVO> iPage = taskService.taskDistributionList(organId, provincial, city,
                street, realName, certiCode, taskName, taskTel, daysRemaining, status, isReturn, area,
                pageNum, pageSize);
        return Result.success(iPage.getRecords());
    }

    /**
     * 评定报告录入列表
     *
     * @param name      失能人员姓名
     * @param certiCode 失能人员身份证号码
     * @param tel       失能人员手机号码
     */
    @RequestMapping("/taskReportList")
    public Result<List<TaskReportVO>> taskReportList(String name, String certiCode, String tel) {
        List<TaskReportVO> taskReportList = taskService.taskReportList(name, certiCode, tel);
        return Result.success(taskReportList);
    }

    /**
     * 通过报告id查询评定报告人员信息
     *
     * @param reportId 评定报告id
     */
    @RequestMapping("/personInfoList")
    public Result<List<PersonInfoVO>> personInfoList(String reportId) {
        List<PersonInfoVO> list = taskService.personInfoList(reportId);
        return Result.success(list);
    }

    /**
     * 查询评定人员列表
     *
     * @param organName   评定机构名称
     * @param name        评定人员姓名
     * @param tel         评定人员手机号码
     * @param groupLeader 是否组长0组长，1非组长
     */
    @RequestMapping("/aprEmpList")
    public Result<List<AprEmpVO>> aprEmpList(String organName, String name, String tel, String groupLeader) {
        List<AprEmpVO> aprEmpS = taskService.aprEmpList(organName, name, tel, groupLeader);
        return Result.success(aprEmpS);
    }

    /**
     * 查询失能等级列表
     */
    @RequestMapping("/cfgSrvDisableLevelList")
    public Result<List<ChCfgSrvDisableLevel>> cfgSrvDisableLevelList() {
        List<ChCfgSrvDisableLevel> list = taskService.cfgSrvDisableLevelList();
        return Result.success(list);
    }

    /**
     * 评定报告新增
     *
     * @param taskId         评定任务id
     * @param person1        鉴定人员1id
     * @param person2        鉴定人员2id
     * @param person3        鉴定人员3id
     * @param evaluateTime   评估时间 格式:2020-11-12
     * @param disableLevelId 失能等级id
     * @param polNo          保单号
     * @param remark         备注
     */
    @RequestMapping("/addReport")
    public Result addReport(Integer taskId, Integer person1, Integer person2, Integer person3, String evaluateTime, Integer disableLevelId, String polNo, String remark) {
        if (person1 == null) {
            return Result.failed("鉴定人员1不能为空!");
        }
        if (taskId == null) {
            return Result.failed("评定任务id不能为空!");
        }
        if (disableLevelId == null) {
            return Result.failed("失能等级id不能为空!");
        }
        if (StringUtils.isBlank(evaluateTime)) {
            return Result.failed("评估时间不能为空!");
        }
        Date evaluateDate;
        try {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            evaluateDate = sdf.parse(evaluateTime);
        } catch (ParseException e) {
            return Result.failed("评估时间格式不正确!");
        }
        Result<Object> objectResult = taskService.addReport(taskId, person1, person2, person3, evaluateDate, disableLevelId, polNo, remark);
        if (!ResultCode.SUCCESS.getCode().equals(objectResult.getCode())) {
            //失败
            return Result.failed(objectResult.getMsg());
        }
        return Result.success();
    }

    /**
     * 评定报告修改
     *
     * @param reportId       评定报告id
     * @param person1        鉴定人员1id
     * @param person2        鉴定人员2id
     * @param person3        鉴定人员3id
     * @param evaluateTime   评估时间 格式:2020-11-12
     * @param disableLevelId 失能等级id
     * @param polNo          保单号
     * @param remark         备注
     */
    @RequestMapping("/updateReport")
    public Result updateReport(Integer reportId, Integer person1, Integer person2, Integer person3, String evaluateTime, Integer disableLevelId, String polNo, String remark) {
        if (disableLevelId == null) {
            return Result.failed("失能等级不能为空!");
        }
        if (polNo == null) {
            polNo = "";
        }
        if (remark == null) {
            remark = "";
        }
        Date evaluateDate;
        try {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            evaluateDate = sdf.parse(evaluateTime);
        } catch (ParseException e) {
            return Result.failed("评估时间格式不正确!");
        }
        taskService.updateReport(reportId, person1, person2, person3, evaluateDate, disableLevelId, polNo, remark);
        return Result.success();
    }

    /**
     * 评定任务分配
     *
     * @param taskId 评定任务id
     * @param empId  评定人员id
     */
    @RequestMapping("/allotTask")
    public Result allotTask(String taskId, String empId) {
        if (StringUtils.isBlank(taskId) && StringUtils.isBlank(empId)) {
            return Result.failed("id不能为空!");
        }
        Result<Object> objectResult = taskService.allotTask(taskId, empId);
        if (!ResultCode.SUCCESS.getCode().equals(objectResult.getCode())) {
            //失败
            return Result.failed(objectResult.getMsg());
        }
        return Result.success();
    }

    /**
     * 分配历史信息列表
     */
    @RequestMapping("/allotTaskHisList")
    public Result<List<AllotTaskHisVO>> allotTaskHisList() {
        List<AllotTaskHisVO> list = taskService.allotTaskHisList();
        return Result.success(list);
    }

    /**
     * 评定任务分组管理列表
     *
     * @param name      评定人员姓名
     * @param certiCode 评定人员身份证号码
     * @param tel       评定人员手机号码
     */
    @RequestMapping("/taskGroupList")
    public Result<List<TaskGroupVO>> taskGroupList(String name, String certiCode, String tel) {
        List<TaskGroupVO> taskReportList = taskService.taskGroupList(name, certiCode, tel);
        return Result.success(taskReportList);
    }

    /**
     * 评定任务分组管理列表详情
     *
     * @param groupNumber 组号
     */
    @RequestMapping("/taskGroupInfoList")
    public Result<List<TaskGroupInfoVO>> taskGroupInfoList(String groupNumber) {
        if (StringUtils.isBlank(groupNumber)) {
            return Result.failed("组号不能为空!");
        }
        List<TaskGroupInfoVO> groupInfoList = taskService.taskGroupInfoList(groupNumber);
        return Result.success(groupInfoList);
    }

    /**
     * 评定任务分组管理列表详情详情
     *
     * @param pushId 任务组id
     */
    @RequestMapping("/taskGroupInfoListInfo")
    public Result<TaskGroupInfoListInfoVO> taskGroupInfoListInfo(String pushId) {
        if (StringUtils.isBlank(pushId)) {
            return Result.failed("任务组id不能为空!");
        }
        TaskGroupInfoListInfoVO taskGroupInfoListInfoVO = taskService.taskGroupInfoListInfo(pushId);
        return Result.success(taskGroupInfoListInfoVO);
    }

    /**
     * 评定任务分组管理(整组操作)
     *
     * @param groupNumber  组号
     * @param status       分组状态 未分配(拒绝(重新分配)):0;已分配:1;已接受:2;已完成:3;拒绝（中止）:4,拒绝（终止）:5
     * @param refuseReason 拒绝原因id
     * @param refuseRemark 拒绝备注
     */
    @RequestMapping("/taskGroupByGroup")
    public Result taskGroupByGroup(String groupNumber, Integer status, Short refuseReason, String refuseRemark) {
        if (StringUtils.isBlank(groupNumber)) {
            return Result.failed("组号不能为空!");
        }
        if (status == null || status < 0 || status > 5) {
            return Result.failed("状态异常!");
        }
        if (status == 4 || status == 5) {
            if (refuseReason == null) {
                return Result.failed("拒绝原因不能为空!");
            }
        }
        Result<Object> objectResult = taskService.taskGroupByGroup(groupNumber, status, refuseReason, refuseRemark);
        if (!ResultCode.SUCCESS.getCode().equals(objectResult.getCode())) {
            //失败
            return Result.failed(objectResult.getMsg());
        }
        return Result.success();
    }

    /**
     * 评定任务分组管理
     *
     * @param pushId       任务组id
     * @param status       分组状态 未分配(拒绝(重新分配)):0;已分配:1;已接受:2;已完成:3;拒绝（中止）:4,拒绝（终止）:5
     * @param refuseReason 拒绝原因id
     * @param refuseRemark 拒绝备注
     */
    @RequestMapping("/taskGroup")
    public Result taskGroup(Integer pushId, Integer status, Short refuseReason, String refuseRemark) {
        if (status == null || status < 0 || status > 5) {
            return Result.failed("状态异常!");
        }
        if (pushId == null) {
            return Result.failed("任务组id不能为空!");
        }
        if (status == 4 || status == 5) {
            if (refuseReason == null) {
                return Result.failed("拒绝原因不能为空!");
            }
        }
        Result<Object> objectResult = taskService.taskGroup(pushId, status, refuseReason, refuseRemark);
        if (!ResultCode.SUCCESS.getCode().equals(objectResult.getCode())) {
            //失败
            return Result.failed(objectResult.getMsg());
        }
        return Result.success();
    }

    /**
     * 查询拒绝原因
     *
     * @return 拒绝原因
     */
    @RequestMapping("/refuseReasonList")
    public Result<List<ChRefuseReason>> refuseReasonList() {
        List<ChRefuseReason> refuseReasons = taskService.refuseReasonList();
        return Result.success(refuseReasons);
    }

    /**
     * 设置预约日期和备注
     *
     * @param pushId 分组id
     * @param time   预约日期 yyyy-MM-dd
     * @param remark 备注
     */
    @RequestMapping("/updatePushTimeAndRemark")
    public Result updatePushTimeAndRemark(String pushId, String time, String remark) {
        if (StringUtils.isBlank(pushId)) {
            return Result.failed("分组id不能为空!");
        }
        if (StringUtils.isBlank(time)) {
            return Result.failed("预约日期不能为空!");
        }
        if (StringUtils.isBlank(remark)) {
            return Result.failed("备注不能为空!");
        }
        Date timeDate;
        try {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            timeDate = sdf.parse(time);
        } catch (ParseException e) {
            return Result.failed("预约日期格式不正确!");
        }
        Result<Object> objectResult = taskService.updatePushTimeAndRemark(pushId, timeDate, remark);
        if (!ResultCode.SUCCESS.getCode().equals(objectResult.getCode())) {
            //失败
            return Result.failed(objectResult.getMsg());
        }
        return Result.success();
    }

}
