package com.hungraim.ltc.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hungraim.ltc.dao.*;
import com.hungraim.ltc.pojo.entity.assessment.*;
import com.hungraim.ltc.pojo.entity.disable.ChCfgSrvDisableLevel;
import com.hungraim.ltc.pojo.entity.disable.ChDisableInfo;
import com.hungraim.ltc.pojo.vo.assessment.*;
import com.hungraim.ltc.service.TaskService;
import com.hungraim.ltc.util.Result;
import com.hungraim.ltc.util.SerialNumberUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * @author czz
 */
@Service
public class TaskServiceImpl implements TaskService {

    @Autowired
    private TaskMapper taskMapper;
    @Autowired
    private AprEmpMapper aprEmpMapper;
    @Autowired
    private AprReportMapper aprReportMapper;
    @Autowired
    private AprPersonInfoMapper personInfoMapper;
    @Autowired
    private TaskPushMapper taskPushMapper;
    @Autowired
    private TaskPushHisMapper taskPushHisMapper;
    @Autowired
    private CfgSrvDisableLevelMapper cfgSrvDisableLevelMapper;
    @Autowired
    private DisableInfoMapper disableInfoMapper;
    @Autowired
    private RefuseReasonMapper refuseReasonMapper;

    /**
     * 评定任务分配列表
     *
     * @param organId       适用机构代码
     * @param provincial    所在省
     * @param city          所在市
     * @param area          所在区域
     * @param street        所在街道
     * @param realName      失能人员姓名
     * @param certiCode     失能人员身份证号码
     * @param taskName      评定人员姓名
     * @param taskTel       评定人员手机号码
     * @param daysRemaining 评定剩余天数
     * @param status        任务状态
     * @param isReturn      是否退回
     * @param pageNum       当前页
     * @param pageSize      每页几条数据
     */
    @Override
    public IPage<TaskDistributionVO> taskDistributionList(String organId, String provincial, String city,
                                                          String street, String realName, String certiCode, String taskName, String taskTel,
                                                          String daysRemaining, String status, String isReturn, String area,
                                                          int pageNum, int pageSize) {
        List<String> organIds = null;
        if (StrUtil.isNotBlank(organId)) {
            organIds = Arrays.asList(organId.split(","));
        }
        return taskMapper.taskDistributionList(new Page<>(pageNum, pageSize), organIds, provincial, city,
                street, realName, certiCode, taskName, taskTel, daysRemaining, status, isReturn, area);
    }

    /**
     * 评定报告录入列表
     *
     * @param name      失能人员姓名
     * @param certiCode 失能人员身份证号码
     * @param tel       失能人员手机号码
     */
    @Override
    public List<TaskReportVO> taskReportList(String name, String certiCode, String tel) {
        return taskMapper.taskReportList(name, certiCode, tel);
    }

    /**
     * 查询评定人员列表
     *
     * @param organName   评定机构名称
     * @param name        评定人员姓名
     * @param tel         评定人员手机号码
     * @param groupLeader 是否组长0组长，1非组长
     */
    @Override
    public List<AprEmpVO> aprEmpList(String organName, String name, String tel, String groupLeader) {
        return taskMapper.aprEmpList(organName, name, tel, groupLeader);
    }

    /**
     * 评定报告新增
     *
     * @param taskId         评定任务id
     * @param person1        鉴定人员1id
     * @param person2        鉴定人员2id
     * @param person3        鉴定人员3id
     * @param evaluateTime   评估时间 格式:2020-11-12
     * @param disableLevelId 失能等级id
     * @param polNo          保单号
     * @param remark         备注
     */
    @Override
    public Result<Object> addReport(Integer taskId, Integer person1, Integer person2, Integer person3, Date evaluateTime, Integer disableLevelId, String polNo, String remark) {
        ChAprTask task = taskMapper.selectById(taskId);
        if (task != null) {
            return Result.failed("已存在的评定任务报告!");
        }
        //插入评定任务
        ChAprReport report = new ChAprReport();
        report.setTaskId(taskId);
        report.setEffTime(new Date());
        report.setRemark(remark);
        report.setDisableLevelId(disableLevelId);
        report.setPolNo(polNo);
        report.setStatus(0);
        report.setEvaluateTime(evaluateTime);
        aprReportMapper.insert(report);
        //插入评定任务关联人员1.2.3(至少一位,最多三位)
        //person1
        ChAprPersonInfo person1Info = new ChAprPersonInfo();
        person1Info.setReportId(report.getReportId());
        person1Info.setEmpId(person1);
        person1Info.setFcd(new Date());
        personInfoMapper.insert(person1Info);
        if (person2 != null) {
            ChAprPersonInfo personInfo = new ChAprPersonInfo();
            personInfo.setReportId(report.getReportId());
            personInfo.setEmpId(person1);
            personInfo.setFcd(new Date());
            personInfoMapper.insert(personInfo);
        }
        if (person3 != null) {
            ChAprPersonInfo personInfo = new ChAprPersonInfo();
            personInfo.setReportId(report.getReportId());
            personInfo.setEmpId(person1);
            personInfo.setFcd(new Date());
            personInfoMapper.insert(personInfo);
        }
        return Result.success();
    }

    /**
     * 评定报告修改
     *
     * @param reportId       评定报告id
     * @param person1        鉴定人员1id
     * @param person2        鉴定人员2id
     * @param person3        鉴定人员3id
     * @param evaluateDate   评估时间
     * @param disableLevelId 失能等级id
     * @param polNo          保单号
     * @param remark         备注
     */
    @Override
    public void updateReport(Integer reportId, Integer person1, Integer person2, Integer person3, Date evaluateDate, Integer disableLevelId, String polNo, String remark) {
        //update鉴定报告
        ChAprReport chAprReport = aprReportMapper.selectById(reportId);
        chAprReport.setDisableLevelId(disableLevelId);
        chAprReport.setPolNo(polNo);
        chAprReport.setRemark(remark);
        chAprReport.setEvaluateTime(evaluateDate);
        aprReportMapper.updateById(chAprReport);
        //修改鉴定报告人员(有就update,没有就insert)
        List<ChAprPersonInfo> personInfos = personInfoMapper.selectList(new QueryWrapper<ChAprPersonInfo>().eq("report_id", reportId).orderByDesc("info_id"));
        if (person1 != null) {
            if (personInfos.size() >= 1) {
                ChAprPersonInfo personInfo = personInfos.get(0);
                personInfo.setEmpId(person1);
                personInfoMapper.updateById(personInfo);
            } else {
                ChAprPersonInfo personInfo = new ChAprPersonInfo();
                personInfo.setReportId(chAprReport.getReportId());
                personInfo.setEmpId(person1);
                personInfo.setFcd(new Date());
                personInfoMapper.insert(personInfo);
            }
        }
        if (person2 != null) {
            if (personInfos.size() >= 2) {
                ChAprPersonInfo personInfo = personInfos.get(1);
                personInfo.setEmpId(person2);
                personInfoMapper.updateById(personInfo);
            } else {
                ChAprPersonInfo personInfo = new ChAprPersonInfo();
                personInfo.setReportId(chAprReport.getReportId());
                personInfo.setEmpId(person2);
                personInfo.setFcd(new Date());
                personInfoMapper.insert(personInfo);
            }
        }
        if (person3 != null) {
            if (personInfos.size() >= 3) {
                ChAprPersonInfo personInfo = personInfos.get(2);
                personInfo.setEmpId(person3);
                personInfoMapper.updateById(personInfo);
            } else {
                ChAprPersonInfo personInfo = new ChAprPersonInfo();
                personInfo.setReportId(chAprReport.getReportId());
                personInfo.setEmpId(person3);
                personInfo.setFcd(new Date());
                personInfoMapper.insert(personInfo);
            }
        }
    }

    /**
     * 评定任务分配
     *
     * @param taskId 评定任务id
     * @param empId  评定人员id
     */
    @Override
    public Result<Object> allotTask(String taskId, String empId) {
        ChAprEmp chAprEmp = aprEmpMapper.selectById(empId);
        if (chAprEmp == null || chAprEmp.getGroupLeader() == null || chAprEmp.getGroupLeader() != 0) {
            //不是组长
            return Result.failed("评定人员必须是组长!");
        }
        String[] taskIds = taskId.split(",");
        //任务分配日期
        Date assignTime = new Date();
        for (String id : taskIds) {
            ChAprTask task = taskMapper.selectById(id);
            if (task == null) {
                return Result.failed("无效的taskId!");
            }
            //更新评定任务信息
            task.setAprEmpId(Integer.parseInt(empId));
            task.setAssignTime(assignTime);
            taskMapper.updateById(task);
            //更新失能人员信息
            ChDisableInfo chDisableInfo = disableInfoMapper.selectById(task.getDisableInfoId());
            chDisableInfo.setLastTaskId(task.getTaskId());
            disableInfoMapper.updateById(chDisableInfo);
            //生成组
            ChAprTaskPush taskPush = new ChAprTaskPush();
            taskPush.setAssignTime(new Date());
            taskPush.setGroupNumber(SerialNumberUtils.getSerialNumber(empId));
            taskPush.setTaskId(Long.parseLong(id));
            taskPush.setEmpId(Long.parseLong(empId));
            taskPush.setStatus((short) 1);
            taskPush.setIsManual((short) 1);
            //生成分组历史
            ChAprTaskPushHis pushHis = new ChAprTaskPushHis();
            BeanUtils.copyProperties(taskPush, pushHis);
            //插入分组数据
            taskPushMapper.insert(taskPush);
            taskPushHisMapper.insert(pushHis);
        }
        return Result.success();
    }

    /**
     * 评定任务分组管理列表
     *
     * @param name      评定人员姓名
     * @param certiCode 评定人员身份证号码
     * @param tel       评定人员手机号码
     */
    @Override
    public List<TaskGroupVO> taskGroupList(String name, String certiCode, String tel) {
        return taskMapper.taskGroupList(name, certiCode, tel);
    }

    /**
     * 查询失能等级列表
     */
    @Override
    public List<ChCfgSrvDisableLevel> cfgSrvDisableLevelList() {
        return cfgSrvDisableLevelMapper.selectList(new QueryWrapper<>());
    }

    /**
     * 通过报告id查询评定报告人员信息
     *
     * @param reportId 评定报告id
     */
    @Override
    public List<PersonInfoVO> personInfoList(String reportId) {
        return taskMapper.personInfoList(reportId);
    }

    /**
     * 评定任务分组管理列表详情
     *
     * @param groupNumber 组号
     */
    @Override
    public List<TaskGroupInfoVO> taskGroupInfoList(String groupNumber) {
        return taskMapper.taskGroupInfoList(groupNumber);
    }

    @Override
    public Result<Object> taskGroup(Integer pushId, Integer status, Short refuseReason, String refuseRemark) {
        ChAprTaskPush chAprTaskPush = taskPushMapper.selectById(pushId);
        if (chAprTaskPush == null) {
            return Result.failed("不存在的组!");
        }
        //更新任务组状态
        chAprTaskPush.setStatus(status.shortValue());
        chAprTaskPush.setRefuseReason(refuseReason);
        chAprTaskPush.setRefuseRemark(refuseRemark);
        taskPushMapper.updateById(chAprTaskPush);
        //更新评定任务状态
        ChAprTask chAprTask = taskMapper.selectById(chAprTaskPush.getTaskId());
        chAprTask.setTaskStatus(status);
        taskMapper.updateById(chAprTask);
        return Result.success();
    }

    /**
     * 评定任务分组管理(整组操作)
     *
     * @param groupNumber  组号
     * @param status       分组状态 未分配(拒绝(重新分配)):0;已分配:1;已接受:2;已完成:3;拒绝（中止）:4,拒绝（终止）:5
     * @param refuseReason 拒绝原因id
     * @param refuseRemark 拒绝备注
     */
    @Override
    public Result<Object> taskGroupByGroup(String groupNumber, Integer status, Short refuseReason, String refuseRemark) {
        List<ChAprTaskPush> pushList = taskPushMapper.selectList(new QueryWrapper<ChAprTaskPush>().eq("group_number", groupNumber));
        if (pushList == null || pushList.size() <= 0) {
            return Result.failed("不存在的组!");
        }
        short status1 = pushList.get(0).getStatus();
        for (ChAprTaskPush push : pushList) {
            if (status1 != push.getStatus()) {
                return Result.failed("该组状态不一致,不可整组操作!");
            }
        }
        for (ChAprTaskPush push : pushList) {
            //更新任务组状态
            push.setStatus(status.shortValue());
            push.setRefuseReason(refuseReason);
            push.setRefuseRemark(refuseRemark);
            taskPushMapper.updateById(push);
            //更新评定任务状态
            ChAprTask chAprTask = taskMapper.selectById(push.getTaskId());
            chAprTask.setTaskStatus(status);
            taskMapper.updateById(chAprTask);
        }
        return Result.success();
    }

    /**
     * 评定任务分组管理列表详情详情
     *
     * @param pushId 任务组id
     */
    @Override
    public TaskGroupInfoListInfoVO taskGroupInfoListInfo(String pushId) {
        return taskMapper.taskGroupInfoListInfo(pushId);
    }

    /**
     * 分配历史信息列表
     */
    @Override
    public List<AllotTaskHisVO> allotTaskHisList() {
        return taskMapper.allotTaskHisList();
    }

    /**
     * 设置预约日期和备注
     *
     * @param pushId   分组id
     * @param timeDate 预约日期
     * @param remark   备注
     */
    @Override
    public Result<Object> updatePushTimeAndRemark(String pushId, Date timeDate, String remark) {
        ChAprTaskPush chAprTaskPush = taskPushMapper.selectById(pushId);
        if (chAprTaskPush == null) {
            return Result.failed("无效的分组id!");
        }
        chAprTaskPush.setTime(timeDate);
        chAprTaskPush.setRefuseRemark(remark);
        int i = taskPushMapper.updateById(chAprTaskPush);
        if (i <= 0) {
            return Result.failed("更新失败!");
        }
        return Result.success();
    }

    @Override
    public List<ChRefuseReason> refuseReasonList() {
        return refuseReasonMapper.selectList(new QueryWrapper<>());
    }
}