package com.hungraim.ltc.governance.controller;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hungraim.ltc.governance.service.DisableAssessmentCriteriaService;
import com.hungraim.ltc.pojo.vo.governance.CfgAprVO;
import com.hungraim.ltc.util.Result;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 失能评定标准Controller
 *
 * @author czz
 */
@RestController
@RequestMapping("/api.governance/resource")
@Slf4j
public class DisableAssessmentCriteriaController {

    @Autowired
    private DisableAssessmentCriteriaService disableAssessmentCriteriaService;

    /**
     * 失能评定标准定义新增/编辑
     *
     * @param jsonParam json参数:
     *                  <p>
     *                  aprId aprId为空就新增,aprId不为空就编辑
     *                  organId 适用机构id
     *                  estimateName 失能评定标准名称
     *                  remark 备注
     *                  status 未生效：0；已生效：1；
     *                  project[]{itemId,projectNama,minVal,maxVal,projectScoreCriteria[]{clauseId,criteria,reference}} 评定项目
     *                  <p>
     *                  projectNama 评定项目名称
     *                  minVal 评定项目最小值
     *                  maxVal 评定项目最大值
     *                  projectScoreCriteria 评定项目评分标准
     *                  criteria 评分标准
     *                  reference 分值参考
     */
    @RequestMapping("/addOrUpdateDisableAssessmentCriteria")
    public Result<Object> addOrUpdateDisableAssessmentCriteria(String jsonParam) {
        //参数校验
        if (StringUtils.isBlank(jsonParam)) {
            return Result.failed("参数不能为空!");
        }
        JSONObject jsonObject = JSONUtil.parseObj(jsonParam);
        String organId = jsonObject.getStr("organId");
        if (StringUtils.isBlank(organId)) {
            return Result.failed("适用机构不能为空!");
        }
        String estimateName = jsonObject.getStr("estimateName");
        if (StringUtils.isBlank(estimateName)) {
            return Result.failed("失能评定标准名称不能为空!");
        }
        Integer status = jsonObject.getInt("status");
        if (status == null || (status != 0 && status != 1)) {
            return Result.failed("状态格式不正确!");
        }
        JSONArray project = jsonObject.getJSONArray("project");
        if (project == null || project.size() <= 0) {
            return Result.failed("评定项目不能为空!");
        }
        for (int i = 0; i < project.size(); i++) {
            JSONObject itemObject = project.getJSONObject(i);
            String projectNama = itemObject.getStr("projectNama");
            String minVal = itemObject.getStr("minVal");
            String maxVal = itemObject.getStr("maxVal");
            short min = Short.parseShort(minVal);
            short max = Short.parseShort(maxVal);
            if (StringUtils.isBlank(projectNama)) {
                return Result.failed("评定项目名称不能为空!");
            }
            if (min > max) {
                return Result.failed("最小值不能大于最大值!");
            }
            JSONArray projectScoreCriteria = project.getJSONObject(i).getJSONArray("projectScoreCriteria");
            if (projectScoreCriteria == null || projectScoreCriteria.size() <= 0) {
                return Result.failed("评定项目评分标准不能为空!");
            }
            for (int j = 0; j < projectScoreCriteria.size(); j++) {
                JSONObject clauseObject = projectScoreCriteria.getJSONObject(j);
                String criteria = clauseObject.getStr("criteria");
                String reference = clauseObject.getStr("reference");
                if (StringUtils.isBlank(criteria)) {
                    return Result.failed("评分标准不能为空!");
                }
                if (StringUtils.isBlank(reference)) {
                    return Result.failed("分值参考不能为空!");
                }
            }
        }
        String remark = jsonObject.getStr("remark");
        Long aprId = jsonObject.getLong("aprId");
        try {
            disableAssessmentCriteriaService.addOrUpdateDisableAssessmentCriteria(aprId, organId, estimateName, remark, status, project);
        } catch (Exception e) {
            //添加失败
            return Result.failed(e.getMessage());
        }
        //添加成功
        return Result.success();
    }

    /**
     * 失能评定标准定义列表
     *
     * @param organIds 机构id
     * @param aprName  评定标准问题模板名称
     * @param status   状态
     */
    @RequestMapping("/getDisableAssessmentCriteriaList")
    public Result<Page<CfgAprVO>> getDisableAssessmentCriteriaList(@RequestParam(defaultValue = "1") Integer pageNum, @RequestParam(defaultValue = "10") Integer pageSize,
                                                                   String organIds, String aprName, String status) {
        Page<CfgAprVO> page = new Page<>(pageNum, pageSize);
        page = disableAssessmentCriteriaService.getDisableAssessmentCriteriaList(page, organIds, aprName, status);
        return Result.success(page);
    }

    /**
     * 停用失能评定标准
     *
     * @param aprId 失能评定标准配置id
     */
    @RequestMapping("/stopDisableAssessmentCriteria")
    public Result<Object> stopDisableAssessmentCriteria(String aprId) {
        disableAssessmentCriteriaService.stopDisableAssessmentCriteria(aprId);
        return Result.success();
    }

}
