package com.hungraim.ltc.governance.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hungraim.ltc.governance.service.SrvTaskService;
import com.hungraim.ltc.pojo.vo.governance.*;
import com.hungraim.ltc.util.DateUtils;
import com.hungraim.ltc.util.Result;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 服务计划Controller
 *
 * @author czz
 */
@RestController
@RequestMapping("/api.governance/srvTask")
@Slf4j
public class SrvTaskController {

    @Autowired
    private SrvTaskService srvTaskService;

    /**
     * 查询护理机构人员列表
     *
     * @param pageNum    页码
     * @param pageSize   每页大小
     * @param srvOrganId 护理服务机构id
     * @param name       护理人员姓名
     * @param certiType  护理人员证件类型
     * @param certifNum  护理人员证件号码
     */
    @RequestMapping("/getSrvOrganEmpList")
    public Result<Page<OrganEmpVO>> getSrvOrganEmpList(@RequestParam(defaultValue = "1") Integer pageNum, @RequestParam(defaultValue = "10") Integer pageSize,
                                                       String srvOrganId, String name, String certiType, String certifNum) {
        if (StringUtils.isBlank(srvOrganId)) {
            return Result.failed("参数异常!");
        }
        Page<OrganEmpVO> page = new Page<>(pageNum, pageSize);
        page = srvTaskService.getSrvOrganEmpList(page, srvOrganId, name, certiType, certifNum);
        return Result.success(page);
    }

    /**
     * 服务计划制定列表
     *
     * @param srvOrganId  护理服务机构id
     * @param disableName 失能人员姓名
     * @param disableTel  失能人员手机号码
     * @param taskStatus  服务计划状态
     */
    @RequestMapping("/getSrvTaskList")
    public Result<Page<SrvTaskVO>> getSrvTaskList(@RequestParam(defaultValue = "1") Integer pageNum, @RequestParam(defaultValue = "10") Integer pageSize,
                                                  Long srvOrganId, String disableName, String disableTel, String taskStatus) {
        Page<SrvTaskVO> page = new Page<>(pageNum, pageSize);
        page = srvTaskService.getSrvTaskList(page, srvOrganId, disableName, disableTel, taskStatus);
        return Result.success(page);
    }

    /**
     * 服务计划查询列表
     *
     * @param srvOrganId  护理机构id
     * @param disableName 失能人员姓名
     * @param disableTel  失能人员手机号码
     * @param empName     护理人员姓名
     * @param certifNum   护理人员证件号码
     * @param startTime   服务计划开始时间
     * @param endTime     服务计划结束时间
     * @param status      任务状态
     * @param modeId      服务方式
     */
    @RequestMapping("/getSrvTaskDetailList")
    public Result<Page<SrvTaskDetailVO>> getSrvTaskDetailList(@RequestParam(defaultValue = "1") Integer pageNum, @RequestParam(defaultValue = "10") Integer pageSize,
                                                              Integer srvOrganId, String disableName, String disableTel,
                                                              String empName, String certifNum, @DateTimeFormat(pattern = "yyyy-MM-dd") Date startTime,@DateTimeFormat(pattern = "yyyy-MM-dd") Date endTime,
                                                              String status, String modeId) {
        Page<SrvTaskDetailVO> page = new Page<>(pageNum, pageSize);
        page = srvTaskService.getSrvTaskDetailList(page, srvOrganId, disableName, disableTel, empName, certifNum, startTime, endTime, status, modeId);
        return Result.success(page);
    }

    /**
     * 查询服务计划制定详情列表
     *
     * @param programId 服务方案id
     */
    @RequestMapping("/getSrvTaskInfo")
    public Result<SrvTaskInfoListVO> getSrvTaskInfoList(String programId) {
        if (StringUtils.isBlank(programId)) {
            return Result.failed("参数异常!");
        }
        List<SrvTaskInfoVO> taskInfos = srvTaskService.getSrvTaskInfoList(programId);
        List<SrvTaskInfoListVO> listVOS = new ArrayList<>();
        SrvTaskInfoListVO taskInfoListVO = new SrvTaskInfoListVO();
        if (taskInfos != null && taskInfos.size() > 0) {
            taskInfoListVO.setProgramId(taskInfos.get(0).getProgramId());
            taskInfoListVO.setDisabInfoId(taskInfos.get(0).getDisabInfoId());
            taskInfoListVO.setTaskList(taskInfos);
        }
        return Result.success(taskInfoListVO);
    }

    /**
     * 新增服务计划
     */
    @RequestMapping("/addSrvTask")
    public Result<List<SrvTaskInfoVO>> addSrvTask(SrvTaskInfoListVO param) {
        String programId = param.getProgramId();
        List<SrvTaskInfoVO> taskList = param.getTaskList();
        if (StringUtils.isBlank(programId) || taskList == null || taskList.size() <= 0) {
            return Result.failed("参数错误!");
        }
        try {
            for (int i = 0; i < taskList.size(); i++) {
                SrvTaskInfoVO srvTaskInfoVO = taskList.get(i);
                if (srvTaskInfoVO.getSrvEmpId() == null) {
                    return Result.failed("护理人员id不能为空!");
                }
                if (srvTaskInfoVO.getTaskDetails() == null || srvTaskInfoVO.getTaskDetails().size() <= 0) {
                    return Result.failed("具体护理日期不能为空!");
                }
                List<Date> dates = new ArrayList<>();
                for (String detail : srvTaskInfoVO.getTaskDetails()) {
                    Date date = DateUtils.strToDate(detail);
                    dates.add(date);
                }
                srvTaskService.addSrvTask(Long.parseLong(programId), Long.parseLong(srvTaskInfoVO.getSrvEmpId()), dates);
            }
        } catch (RuntimeException e) {
            //添加失败
            return Result.failed(e.getMessage());
        }
        //添加成功
        List<SrvTaskInfoVO> taskInfos = srvTaskService.getSrvTaskInfoList(programId);
        return Result.success(taskInfos);
    }

    /**
     * 修改服务计划
     */
    @RequestMapping("/updateSrvTask")
    public Result<Object> updateSrvTask(SrvTaskInfoListVO param) {
        String programId = param.getProgramId();
        List<SrvTaskInfoVO> taskList = param.getTaskList();
        if (StringUtils.isBlank(programId) || taskList == null || taskList.size() <= 0) {
            return Result.failed("参数错误!");
        }
        try {
            for (int i = 0; i < taskList.size(); i++) {
                String taskId = taskList.get(i).getSrvTaskId();
                String srvEmpId = taskList.get(i).getSrvEmpId();
                List<String> taskDetails = taskList.get(i).getTaskDetails();
                if (StringUtils.isBlank(taskId)) {
                    return Result.failed("taskId不能为空!");
                }
                if (StringUtils.isBlank(srvEmpId)) {
                    return Result.failed("护理人员id不能为空!");
                }
                if (taskDetails == null || taskDetails.size() <= 0) {
                    return Result.failed("具体护理日期不能为空!");
                }
                List<Date> dates = new ArrayList<>();
                for (String detail : taskDetails) {
                    Date date = DateUtils.strToDate(detail);
                    dates.add(date);
                }
                srvTaskService.updateSrvTask(taskId, Long.parseLong(srvEmpId), dates);
            }
        } catch (RuntimeException e) {
            //修改失败
            return Result.failed(e.getMessage());
        }
        //修改成功
        List<SrvTaskInfoVO> taskInfos = srvTaskService.getSrvTaskInfoList(programId);
        return Result.success(taskInfos);
    }
}
